import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:get/get.dart';
import 'package:mirrorfly_plugin/mirrorflychat.dart';
import 'package:mirrorfly_uikit_plugin/app/call_modules/call_widgets.dart';

import '../../common/constants.dart';
import '../ripple_animation_view.dart';
import 'call_controller.dart';

class OutGoingCallView extends StatefulWidget {
  const OutGoingCallView({Key? key,  required this.userJid}) : super(key: key);

  final String userJid;

  @override
  State<OutGoingCallView> createState() => _OutGoingCallViewState();
}

class _OutGoingCallViewState extends State<OutGoingCallView> {
  final controller = Get.put(CallController());
  @override
  void initState() {
    super.initState();
    controller.initCallController(buildContext: context, userJid: widget.userJid);
  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: AppColors.callerBackground,
      body: SafeArea(
        child: InkWell(
          splashColor: Colors.transparent,
          highlightColor: Colors.transparent,
          onTap: () {
            debugPrint("clicked");
            controller.isVisible(!controller.isVisible.value);
          },
          child: Stack(
            children: [
              Column(
                children: [
                  Expanded(
                      child: Column(
                    children: [
                      const SizedBox(
                        height: 10,
                      ),
                      Obx(() {
                        return Text(
                          controller.callStatus.value,
                          style: const TextStyle(
                              color: AppColors.callerStatus,
                              fontWeight: FontWeight.w100,
                              fontSize: 14),
                        );
                      }),
                      const SizedBox(
                        height: 16,
                      ),
                      Obx(() {
                        return Text(
                          controller.calleeName.value,
                          style: const TextStyle(
                              color: Colors.white,
                              fontWeight: FontWeight.w500,
                              fontSize: 18),
                          overflow: TextOverflow.ellipsis,
                        );
                      }),
                      Obx(() {
                        return RipplesAnimation(
                          onPressed: () {},
                          child: buildProfileImage(controller.profile.value),
                        );
                      }),
                    ],
                  )),
                  Obx(() {
                    debugPrint("audio item changed ${controller.audioOutputType.value}");
                    return Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 50),
                      child: Row(
                        mainAxisSize: MainAxisSize.max,
                        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                        children: [
                          controller.muted.value
                              ? FloatingActionButton(
                                  heroTag: "mute",
                                  elevation: 0,
                                  backgroundColor: Colors.white,
                                  onPressed: () => controller.muteAudio(),
                                  child: SvgPicture.asset(
                                    muteActive,package: package,
                                  ),
                                )
                              : FloatingActionButton(
                                  heroTag: "mute",
                                  elevation: 0,
                                  backgroundColor:
                                      Colors.white.withOpacity(0.3),
                                  onPressed: () => controller.muteAudio(),
                                  child: SvgPicture.asset(
                                    muteInactive,package: package,
                                  ),
                                ),
                          FloatingActionButton(
                            heroTag: "video",
                            elevation: 0,
                            backgroundColor: controller.videoMuted.value
                                ? Colors.white
                                : Colors.white.withOpacity(0.3),
                            onPressed: () => controller.videoMute(),
                            child: controller.videoMuted.value
                                ? SvgPicture.asset(videoInactive,package: package,)
                                : SvgPicture.asset(videoActive,package: package,),
                          ),
                          FloatingActionButton(
                            heroTag: "speaker",
                            elevation: 0,
                            backgroundColor:
                            controller.audioOutputType.value == AudioDeviceType.receiver
                                ? Colors.white.withOpacity(0.3)
                                : Colors.white,
                            onPressed: () => controller.changeSpeaker(context),
                            child: controller.audioOutputType.value == AudioDeviceType.receiver
                                ? SvgPicture.asset(speakerInactive,package: package,)
                                : controller.audioOutputType.value == AudioDeviceType.speaker
                                ? SvgPicture.asset(speakerActive,package: package,)
                                : controller.audioOutputType.value ==
                                AudioDeviceType.bluetooth
                                ? SvgPicture.asset(speakerBluetooth,package: package,)
                                : controller.audioOutputType.value ==
                                AudioDeviceType.headset
                                ? SvgPicture.asset(speakerHeadset,package: package,)
                                : SvgPicture.asset(speakerActive,package: package,),
                          ),
                        ],
                      ),
                    );
                  }),
                  Padding(
                    padding: const EdgeInsets.only(bottom: 10.0, top: 20.0),
                    child: ElevatedButton(
                        style: ElevatedButton.styleFrom(
                            padding: EdgeInsets.zero,
                            maximumSize: const Size(200, 50),
                            shape: const StadiumBorder(),
                            backgroundColor: AppColors.endButton),
                        onPressed: () {
                          controller.declineCall();
                        },
                        child: SvgPicture.asset(
                          callEndButton,package: package,
                        )),
                  ),
                ],
              ),
              /*Obx(() {
                return AnimatedPositioned(
                  duration: const Duration(milliseconds: 500),
                  curve: Curves.easeInOut,
                  bottom: controller.isVisible.value ? 0.0 : -200,
                  left: 0.0,
                  right: 0.0,
                  height: 200,
                  child: Container(
                    color: Colors.blue,
                    child: const Center(
                      child: Text(
                        'Slide Animation',
                        style: TextStyle(
                          color: Colors.white,
                          fontSize: 20.0,
                        ),
                      ),
                    ),
                  ),
                );
              }),
              Obx(() {
                return AnimatedPositioned(
                  duration: const Duration(milliseconds: 500),
                  curve: Curves.easeInOut,
                  top: controller.isVisible.value ? 0.0 : -72,
                  left: 0.0,
                  right: 0.0,
                  height: 72,
                  child: Container(
                    color: Colors.blue,
                    child: const Center(
                      child: Text(
                        'Slide Animation',
                        style: TextStyle(
                          color: Colors.white,
                          fontSize: 20.0,
                        ),
                      ),
                    ),
                  ),
                );
              }),*/
            ],
          ),
        ),
      ),
    );
  }
}
